# -*- coding: utf-8 -*-
# Copyright (c) 2013 Spotify AB
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.

# python 3 support
from __future__ import absolute_import, print_function, division

from snakebite.channel import SocketRpcChannel
import google.protobuf.service as service


class RpcService(object):
    def __init__(self, service_stub_class, port, host, hadoop_version, effective_user=None,
                 use_sasl=False, hdfs_namenode_principal=None, sock_connect_timeout=10000,
                 sock_request_timeout=10000):
        self.service_stub_class = service_stub_class
        self.port = port
        self.host = host

        # Setup the RPC channel
        self.channel = SocketRpcChannel(host=self.host, port=self.port, version=hadoop_version,
                                        effective_user=effective_user, use_sasl=use_sasl,
                                        hdfs_namenode_principal=hdfs_namenode_principal,
                                        sock_connect_timeout=sock_connect_timeout,
                                        sock_request_timeout=sock_request_timeout,)
        self.service = self.service_stub_class(self.channel)

        # go through service_stub methods and add a wrapper function to
        # this object that will call the method
        for method in service_stub_class.GetDescriptor().methods:
            # Add service methods to the this object
            rpc = lambda request, service=self, method=method.name: service.call(service_stub_class.__dict__[method], request)

            self.__dict__[method.name] = rpc

    def call(self, method, request):
        controller = SocketRpcController()
        return method(self.service, controller, request)


class SocketRpcController(service.RpcController):
    ''' RpcController implementation to be used by the SocketRpcChannel class.

    The RpcController is used to mediate a single method call.
    '''

    def __init__(self):
        '''Constructor which initializes the controller's state.'''
        self._fail = False
        self._error = None
        self.reason = None

    def handleError(self, error_code, message):
        '''Log and set the controller state.'''
        self._fail = True
        self.reason = error_code
        self._error = message

    def reset(self):
        '''Resets the controller i.e. clears the error state.'''
        self._fail = False
        self._error = None
        self.reason = None

    def failed(self):
        '''Returns True if the controller is in a failed state.'''
        return self._fail

    def error(self):
        return self._error



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ProtobufRpcEngine.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ProtobufRpcEngine.proto',
  package='hadoop.common',
  syntax='proto2',
  serialized_pb=_b('\n\x17ProtobufRpcEngine.proto\x12\rhadoop.common\"k\n\x12RequestHeaderProto\x12\x12\n\nmethodName\x18\x01 \x02(\t\x12\"\n\x1a\x64\x65\x63laringClassProtocolName\x18\x02 \x02(\t\x12\x1d\n\x15\x63lientProtocolVersion\x18\x03 \x02(\x04\x42<\n\x1eorg.apache.hadoop.ipc.protobufB\x17ProtobufRpcEngineProtos\xa0\x01\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_REQUESTHEADERPROTO = _descriptor.Descriptor(
  name='RequestHeaderProto',
  full_name='hadoop.common.RequestHeaderProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='methodName', full_name='hadoop.common.RequestHeaderProto.methodName', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='declaringClassProtocolName', full_name='hadoop.common.RequestHeaderProto.declaringClassProtocolName', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clientProtocolVersion', full_name='hadoop.common.RequestHeaderProto.clientProtocolVersion', index=2,
      number=3, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=42,
  serialized_end=149,
)

DESCRIPTOR.message_types_by_name['RequestHeaderProto'] = _REQUESTHEADERPROTO

RequestHeaderProto = _reflection.GeneratedProtocolMessageType('RequestHeaderProto', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTHEADERPROTO,
  __module__ = 'ProtobufRpcEngine_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.common.RequestHeaderProto)
  ))
_sym_db.RegisterMessage(RequestHeaderProto)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\036org.apache.hadoop.ipc.protobufB\027ProtobufRpcEngineProtos\240\001\001'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: datatransfer.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Security_pb2 as Security__pb2
import hdfs_pb2 as hdfs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='datatransfer.proto',
  package='hadoop.hdfs',
  syntax='proto2',
  serialized_pb=_b('\n\x12\x64\x61tatransfer.proto\x12\x0bhadoop.hdfs\x1a\x0eSecurity.proto\x1a\nhdfs.proto\"\xef\x01\n!DataTransferEncryptorMessageProto\x12Z\n\x06status\x18\x01 \x02(\x0e\x32J.hadoop.hdfs.DataTransferEncryptorMessageProto.DataTransferEncryptorStatus\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\x12\x0f\n\x07message\x18\x03 \x01(\t\"L\n\x1b\x44\x61taTransferEncryptorStatus\x12\x0b\n\x07SUCCESS\x10\x00\x12\x15\n\x11\x45RROR_UNKNOWN_KEY\x10\x01\x12\t\n\x05\x45RROR\x10\x02\"k\n\x0f\x42\x61seHeaderProto\x12.\n\x05\x62lock\x18\x01 \x02(\x0b\x32\x1f.hadoop.hdfs.ExtendedBlockProto\x12(\n\x05token\x18\x02 \x01(\x0b\x32\x19.hadoop.common.TokenProto\"b\n\x1a\x43lientOperationHeaderProto\x12\x30\n\nbaseHeader\x18\x01 \x02(\x0b\x32\x1c.hadoop.hdfs.BaseHeaderProto\x12\x12\n\nclientName\x18\x02 \x02(\t\"=\n\x14\x43\x61\x63hingStrategyProto\x12\x12\n\ndropBehind\x18\x01 \x01(\x08\x12\x11\n\treadahead\x18\x02 \x01(\x03\"\xc1\x01\n\x10OpReadBlockProto\x12\x37\n\x06header\x18\x01 \x02(\x0b\x32\'.hadoop.hdfs.ClientOperationHeaderProto\x12\x0e\n\x06offset\x18\x02 \x02(\x04\x12\x0b\n\x03len\x18\x03 \x02(\x04\x12\x1b\n\rsendChecksums\x18\x04 \x01(\x08:\x04true\x12:\n\x0f\x63\x61\x63hingStrategy\x18\x05 \x01(\x0b\x32!.hadoop.hdfs.CachingStrategyProto\"W\n\rChecksumProto\x12,\n\x04type\x18\x01 \x02(\x0e\x32\x1e.hadoop.hdfs.ChecksumTypeProto\x12\x18\n\x10\x62ytesPerChecksum\x18\x02 \x02(\r\"\xd2\x05\n\x11OpWriteBlockProto\x12\x37\n\x06header\x18\x01 \x02(\x0b\x32\'.hadoop.hdfs.ClientOperationHeaderProto\x12/\n\x07targets\x18\x02 \x03(\x0b\x32\x1e.hadoop.hdfs.DatanodeInfoProto\x12.\n\x06source\x18\x03 \x01(\x0b\x32\x1e.hadoop.hdfs.DatanodeInfoProto\x12\x44\n\x05stage\x18\x04 \x02(\x0e\x32\x35.hadoop.hdfs.OpWriteBlockProto.BlockConstructionStage\x12\x14\n\x0cpipelineSize\x18\x05 \x02(\r\x12\x14\n\x0cminBytesRcvd\x18\x06 \x02(\x04\x12\x14\n\x0cmaxBytesRcvd\x18\x07 \x02(\x04\x12\x1d\n\x15latestGenerationStamp\x18\x08 \x02(\x04\x12\x35\n\x11requestedChecksum\x18\t \x02(\x0b\x32\x1a.hadoop.hdfs.ChecksumProto\x12:\n\x0f\x63\x61\x63hingStrategy\x18\n \x01(\x0b\x32!.hadoop.hdfs.CachingStrategyProto\"\x88\x02\n\x16\x42lockConstructionStage\x12\x19\n\x15PIPELINE_SETUP_APPEND\x10\x00\x12\"\n\x1ePIPELINE_SETUP_APPEND_RECOVERY\x10\x01\x12\x12\n\x0e\x44\x41TA_STREAMING\x10\x02\x12%\n!PIPELINE_SETUP_STREAMING_RECOVERY\x10\x03\x12\x12\n\x0ePIPELINE_CLOSE\x10\x04\x12\x1b\n\x17PIPELINE_CLOSE_RECOVERY\x10\x05\x12\x19\n\x15PIPELINE_SETUP_CREATE\x10\x06\x12\x10\n\x0cTRANSFER_RBW\x10\x07\x12\x16\n\x12TRANSFER_FINALIZED\x10\x08\"\x80\x01\n\x14OpTransferBlockProto\x12\x37\n\x06header\x18\x01 \x02(\x0b\x32\'.hadoop.hdfs.ClientOperationHeaderProto\x12/\n\x07targets\x18\x02 \x03(\x0b\x32\x1e.hadoop.hdfs.DatanodeInfoProto\"\x84\x01\n\x13OpReplaceBlockProto\x12,\n\x06header\x18\x01 \x02(\x0b\x32\x1c.hadoop.hdfs.BaseHeaderProto\x12\x0f\n\x07\x64\x65lHint\x18\x02 \x02(\t\x12.\n\x06source\x18\x03 \x02(\x0b\x32\x1e.hadoop.hdfs.DatanodeInfoProto\"@\n\x10OpCopyBlockProto\x12,\n\x06header\x18\x01 \x02(\x0b\x32\x1c.hadoop.hdfs.BaseHeaderProto\"D\n\x14OpBlockChecksumProto\x12,\n\x06header\x18\x01 \x02(\x0b\x32\x1c.hadoop.hdfs.BaseHeaderProto\"d\n OpRequestShortCircuitAccessProto\x12,\n\x06header\x18\x01 \x02(\x0b\x32\x1c.hadoop.hdfs.BaseHeaderProto\x12\x12\n\nmaxVersion\x18\x02 \x02(\r\"\x7f\n\x11PacketHeaderProto\x12\x15\n\roffsetInBlock\x18\x01 \x02(\x10\x12\r\n\x05seqno\x18\x02 \x02(\x10\x12\x19\n\x11lastPacketInBlock\x18\x03 \x02(\x08\x12\x0f\n\x07\x64\x61taLen\x18\x04 \x02(\x0f\x12\x18\n\tsyncBlock\x18\x05 \x01(\x08:\x05\x66\x61lse\"i\n\x10PipelineAckProto\x12\r\n\x05seqno\x18\x01 \x02(\x12\x12#\n\x06status\x18\x02 \x03(\x0e\x32\x13.hadoop.hdfs.Status\x12!\n\x16\x64ownstreamAckTimeNanos\x18\x03 \x01(\x04:\x01\x30\"\\\n\x17ReadOpChecksumInfoProto\x12,\n\x08\x63hecksum\x18\x01 \x02(\x0b\x32\x1a.hadoop.hdfs.ChecksumProto\x12\x13\n\x0b\x63hunkOffset\x18\x02 \x02(\x04\"\x8c\x02\n\x14\x42lockOpResponseProto\x12#\n\x06status\x18\x01 \x02(\x0e\x32\x13.hadoop.hdfs.Status\x12\x14\n\x0c\x66irstBadLink\x18\x02 \x01(\t\x12\x43\n\x10\x63hecksumResponse\x18\x03 \x01(\x0b\x32).hadoop.hdfs.OpBlockChecksumResponseProto\x12@\n\x12readOpChecksumInfo\x18\x04 \x01(\x0b\x32$.hadoop.hdfs.ReadOpChecksumInfoProto\x12\x0f\n\x07message\x18\x05 \x01(\t\x12!\n\x19shortCircuitAccessVersion\x18\x06 \x01(\r\"<\n\x15\x43lientReadStatusProto\x12#\n\x06status\x18\x01 \x02(\x0e\x32\x13.hadoop.hdfs.Status\"9\n\x12\x44NTransferAckProto\x12#\n\x06status\x18\x01 \x02(\x0e\x32\x13.hadoop.hdfs.Status\"\x86\x01\n\x1cOpBlockChecksumResponseProto\x12\x13\n\x0b\x62ytesPerCrc\x18\x01 \x02(\r\x12\x13\n\x0b\x63rcPerBlock\x18\x02 \x02(\x04\x12\x0b\n\x03md5\x18\x03 \x02(\x0c\x12/\n\x07\x63rcType\x18\x04 \x01(\x0e\x32\x1e.hadoop.hdfs.ChecksumTypeProto*\x99\x01\n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x12\n\x0e\x45RROR_CHECKSUM\x10\x02\x12\x11\n\rERROR_INVALID\x10\x03\x12\x10\n\x0c\x45RROR_EXISTS\x10\x04\x12\x16\n\x12\x45RROR_ACCESS_TOKEN\x10\x05\x12\x0f\n\x0b\x43HECKSUM_OK\x10\x06\x12\x15\n\x11\x45RROR_UNSUPPORTED\x10\x07\x42>\n%org.apache.hadoop.hdfs.protocol.protoB\x12\x44\x61taTransferProtos\xa0\x01\x01')
  ,
  dependencies=[Security__pb2.DESCRIPTOR,hdfs__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='hadoop.hdfs.Status',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_CHECKSUM', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_INVALID', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_EXISTS', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_ACCESS_TOKEN', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHECKSUM_OK', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_UNSUPPORTED', index=7, number=7,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2951,
  serialized_end=3104,
)
_sym_db.RegisterEnumDescriptor(_STATUS)

Status = enum_type_wrapper.EnumTypeWrapper(_STATUS)
SUCCESS = 0
ERROR = 1
ERROR_CHECKSUM = 2
ERROR_INVALID = 3
ERROR_EXISTS = 4
ERROR_ACCESS_TOKEN = 5
CHECKSUM_OK = 6
ERROR_UNSUPPORTED = 7


_DATATRANSFERENCRYPTORMESSAGEPROTO_DATATRANSFERENCRYPTORSTATUS = _descriptor.EnumDescriptor(
  name='DataTransferEncryptorStatus',
  full_name='hadoop.hdfs.DataTransferEncryptorMessageProto.DataTransferEncryptorStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR_UNKNOWN_KEY', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=227,
  serialized_end=303,
)
_sym_db.RegisterEnumDescriptor(_DATATRANSFERENCRYPTORMESSAGEPROTO_DATATRANSFERENCRYPTORSTATUS)

_OPWRITEBLOCKPROTO_BLOCKCONSTRUCTIONSTAGE = _descriptor.EnumDescriptor(
  name='BlockConstructionStage',
  full_name='hadoop.hdfs.OpWriteBlockProto.BlockConstructionStage',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_SETUP_APPEND', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_SETUP_APPEND_RECOVERY', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DATA_STREAMING', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_SETUP_STREAMING_RECOVERY', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_CLOSE', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_CLOSE_RECOVERY', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PIPELINE_SETUP_CREATE', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRANSFER_RBW', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRANSFER_FINALIZED', index=8, number=8,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1321,
  serialized_end=1585,
)
_sym_db.RegisterEnumDescriptor(_OPWRITEBLOCKPROTO_BLOCKCONSTRUCTIONSTAGE)


_DATATRANSFERENCRYPTORMESSAGEPROTO = _descriptor.Descriptor(
  name='DataTransferEncryptorMessageProto',
  full_name='hadoop.hdfs.DataTransferEncryptorMessageProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='hadoop.hdfs.DataTransferEncryptorMessageProto.status', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='payload', full_name='hadoop.hdfs.DataTransferEncryptorMessageProto.payload', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='hadoop.hdfs.DataTransferEncryptorMessageProto.message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DATATRANSFERENCRYPTORMESSAGEPROTO_DATATRANSFERENCRYPTORSTATUS,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=303,
)


_BASEHEADERPROTO = _descriptor.Descriptor(
  name='BaseHeaderProto',
  full_name='hadoop.hdfs.BaseHeaderProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='block', full_name='hadoop.hdfs.BaseHeaderProto.block', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='token', full_name='hadoop.hdfs.BaseHeaderProto.token', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=412,
)


_CLIENTOPERATIONHEADERPROTO = _descriptor.Descriptor(
  name='ClientOperationHeaderProto',
  full_name='hadoop.hdfs.ClientOperationHeaderProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='baseHeader', full_name='hadoop.hdfs.ClientOperationHeaderProto.baseHeader', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clientName', full_name='hadoop.hdfs.ClientOperationHeaderProto.clientName', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=414,
  serialized_end=512,
)


_CACHINGSTRATEGYPROTO = _descriptor.Descriptor(
  name='CachingStrategyProto',
  full_name='hadoop.hdfs.CachingStrategyProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dropBehind', full_name='hadoop.hdfs.CachingStrategyProto.dropBehind', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='readahead', full_name='hadoop.hdfs.CachingStrategyProto.readahead', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=514,
  serialized_end=575,
)


_OPREADBLOCKPROTO = _descriptor.Descriptor(
  name='OpReadBlockProto',
  full_name='hadoop.hdfs.OpReadBlockProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpReadBlockProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='offset', full_name='hadoop.hdfs.OpReadBlockProto.offset', index=1,
      number=2, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='len', full_name='hadoop.hdfs.OpReadBlockProto.len', index=2,
      number=3, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sendChecksums', full_name='hadoop.hdfs.OpReadBlockProto.sendChecksums', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cachingStrategy', full_name='hadoop.hdfs.OpReadBlockProto.cachingStrategy', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=578,
  serialized_end=771,
)


_CHECKSUMPROTO = _descriptor.Descriptor(
  name='ChecksumProto',
  full_name='hadoop.hdfs.ChecksumProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='hadoop.hdfs.ChecksumProto.type', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bytesPerChecksum', full_name='hadoop.hdfs.ChecksumProto.bytesPerChecksum', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=773,
  serialized_end=860,
)


_OPWRITEBLOCKPROTO = _descriptor.Descriptor(
  name='OpWriteBlockProto',
  full_name='hadoop.hdfs.OpWriteBlockProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpWriteBlockProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='targets', full_name='hadoop.hdfs.OpWriteBlockProto.targets', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='source', full_name='hadoop.hdfs.OpWriteBlockProto.source', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='stage', full_name='hadoop.hdfs.OpWriteBlockProto.stage', index=3,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pipelineSize', full_name='hadoop.hdfs.OpWriteBlockProto.pipelineSize', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='minBytesRcvd', full_name='hadoop.hdfs.OpWriteBlockProto.minBytesRcvd', index=5,
      number=6, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='maxBytesRcvd', full_name='hadoop.hdfs.OpWriteBlockProto.maxBytesRcvd', index=6,
      number=7, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='latestGenerationStamp', full_name='hadoop.hdfs.OpWriteBlockProto.latestGenerationStamp', index=7,
      number=8, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='requestedChecksum', full_name='hadoop.hdfs.OpWriteBlockProto.requestedChecksum', index=8,
      number=9, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cachingStrategy', full_name='hadoop.hdfs.OpWriteBlockProto.cachingStrategy', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OPWRITEBLOCKPROTO_BLOCKCONSTRUCTIONSTAGE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=863,
  serialized_end=1585,
)


_OPTRANSFERBLOCKPROTO = _descriptor.Descriptor(
  name='OpTransferBlockProto',
  full_name='hadoop.hdfs.OpTransferBlockProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpTransferBlockProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='targets', full_name='hadoop.hdfs.OpTransferBlockProto.targets', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1588,
  serialized_end=1716,
)


_OPREPLACEBLOCKPROTO = _descriptor.Descriptor(
  name='OpReplaceBlockProto',
  full_name='hadoop.hdfs.OpReplaceBlockProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpReplaceBlockProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='delHint', full_name='hadoop.hdfs.OpReplaceBlockProto.delHint', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='source', full_name='hadoop.hdfs.OpReplaceBlockProto.source', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1719,
  serialized_end=1851,
)


_OPCOPYBLOCKPROTO = _descriptor.Descriptor(
  name='OpCopyBlockProto',
  full_name='hadoop.hdfs.OpCopyBlockProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpCopyBlockProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1853,
  serialized_end=1917,
)


_OPBLOCKCHECKSUMPROTO = _descriptor.Descriptor(
  name='OpBlockChecksumProto',
  full_name='hadoop.hdfs.OpBlockChecksumProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpBlockChecksumProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1919,
  serialized_end=1987,
)


_OPREQUESTSHORTCIRCUITACCESSPROTO = _descriptor.Descriptor(
  name='OpRequestShortCircuitAccessProto',
  full_name='hadoop.hdfs.OpRequestShortCircuitAccessProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='hadoop.hdfs.OpRequestShortCircuitAccessProto.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='maxVersion', full_name='hadoop.hdfs.OpRequestShortCircuitAccessProto.maxVersion', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1989,
  serialized_end=2089,
)


_PACKETHEADERPROTO = _descriptor.Descriptor(
  name='PacketHeaderProto',
  full_name='hadoop.hdfs.PacketHeaderProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='offsetInBlock', full_name='hadoop.hdfs.PacketHeaderProto.offsetInBlock', index=0,
      number=1, type=16, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='seqno', full_name='hadoop.hdfs.PacketHeaderProto.seqno', index=1,
      number=2, type=16, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lastPacketInBlock', full_name='hadoop.hdfs.PacketHeaderProto.lastPacketInBlock', index=2,
      number=3, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataLen', full_name='hadoop.hdfs.PacketHeaderProto.dataLen', index=3,
      number=4, type=15, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='syncBlock', full_name='hadoop.hdfs.PacketHeaderProto.syncBlock', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2091,
  serialized_end=2218,
)


_PIPELINEACKPROTO = _descriptor.Descriptor(
  name='PipelineAckProto',
  full_name='hadoop.hdfs.PipelineAckProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seqno', full_name='hadoop.hdfs.PipelineAckProto.seqno', index=0,
      number=1, type=18, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status', full_name='hadoop.hdfs.PipelineAckProto.status', index=1,
      number=2, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='downstreamAckTimeNanos', full_name='hadoop.hdfs.PipelineAckProto.downstreamAckTimeNanos', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2220,
  serialized_end=2325,
)


_READOPCHECKSUMINFOPROTO = _descriptor.Descriptor(
  name='ReadOpChecksumInfoProto',
  full_name='hadoop.hdfs.ReadOpChecksumInfoProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='checksum', full_name='hadoop.hdfs.ReadOpChecksumInfoProto.checksum', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='chunkOffset', full_name='hadoop.hdfs.ReadOpChecksumInfoProto.chunkOffset', index=1,
      number=2, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2327,
  serialized_end=2419,
)


_BLOCKOPRESPONSEPROTO = _descriptor.Descriptor(
  name='BlockOpResponseProto',
  full_name='hadoop.hdfs.BlockOpResponseProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='hadoop.hdfs.BlockOpResponseProto.status', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='firstBadLink', full_name='hadoop.hdfs.BlockOpResponseProto.firstBadLink', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='checksumResponse', full_name='hadoop.hdfs.BlockOpResponseProto.checksumResponse', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='readOpChecksumInfo', full_name='hadoop.hdfs.BlockOpResponseProto.readOpChecksumInfo', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='hadoop.hdfs.BlockOpResponseProto.message', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='shortCircuitAccessVersion', full_name='hadoop.hdfs.BlockOpResponseProto.shortCircuitAccessVersion', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2422,
  serialized_end=2690,
)


_CLIENTREADSTATUSPROTO = _descriptor.Descriptor(
  name='ClientReadStatusProto',
  full_name='hadoop.hdfs.ClientReadStatusProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='hadoop.hdfs.ClientReadStatusProto.status', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2692,
  serialized_end=2752,
)


_DNTRANSFERACKPROTO = _descriptor.Descriptor(
  name='DNTransferAckProto',
  full_name='hadoop.hdfs.DNTransferAckProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='hadoop.hdfs.DNTransferAckProto.status', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2754,
  serialized_end=2811,
)


_OPBLOCKCHECKSUMRESPONSEPROTO = _descriptor.Descriptor(
  name='OpBlockChecksumResponseProto',
  full_name='hadoop.hdfs.OpBlockChecksumResponseProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bytesPerCrc', full_name='hadoop.hdfs.OpBlockChecksumResponseProto.bytesPerCrc', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crcPerBlock', full_name='hadoop.hdfs.OpBlockChecksumResponseProto.crcPerBlock', index=1,
      number=2, type=4, cpp_type=4, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='md5', full_name='hadoop.hdfs.OpBlockChecksumResponseProto.md5', index=2,
      number=3, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crcType', full_name='hadoop.hdfs.OpBlockChecksumResponseProto.crcType', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2814,
  serialized_end=2948,
)

_DATATRANSFERENCRYPTORMESSAGEPROTO.fields_by_name['status'].enum_type = _DATATRANSFERENCRYPTORMESSAGEPROTO_DATATRANSFERENCRYPTORSTATUS
_DATATRANSFERENCRYPTORMESSAGEPROTO_DATATRANSFERENCRYPTORSTATUS.containing_type = _DATATRANSFERENCRYPTORMESSAGEPROTO
_BASEHEADERPROTO.fields_by_name['block'].message_type = hdfs__pb2._EXTENDEDBLOCKPROTO
_BASEHEADERPROTO.fields_by_name['token'].message_type = Security__pb2._TOKENPROTO
_CLIENTOPERATIONHEADERPROTO.fields_by_name['baseHeader'].message_type = _BASEHEADERPROTO
_OPREADBLOCKPROTO.fields_by_name['header'].message_type = _CLIENTOPERATIONHEADERPROTO
_OPREADBLOCKPROTO.fields_by_name['cachingStrategy'].message_type = _CACHINGSTRATEGYPROTO
_CHECKSUMPROTO.fields_by_name['type'].enum_type = hdfs__pb2._CHECKSUMTYPEPROTO
_OPWRITEBLOCKPROTO.fields_by_name['header'].message_type = _CLIENTOPERATIONHEADERPROTO
_OPWRITEBLOCKPROTO.fields_by_name['targets'].message_type = hdfs__pb2._DATANODEINFOPROTO
_OPWRITEBLOCKPROTO.fields_by_name['source'].message_type = hdfs__pb2._DATANODEINFOPROTO
_OPWRITEBLOCKPROTO.fields_by_name['stage'].enum_type = _OPWRITEBLOCKPROTO_BLOCKCONSTRUCTIONSTAGE
_OPWRITEBLOCKPROTO.fields_by_name['requestedChecksum'].message_type = _CHECKSUMPROTO
_OPWRITEBLOCKPROTO.fields_by_name['cachingStrategy'].message_type = _CACHINGSTRATEGYPROTO
_OPWRITEBLOCKPROTO_BLOCKCONSTRUCTIONSTAGE.containing_type = _OPWRITEBLOCKPROTO
_OPTRANSFERBLOCKPROTO.fields_by_name['header'].message_type = _CLIENTOPERATIONHEADERPROTO
_OPTRANSFERBLOCKPROTO.fields_by_name['targets'].message_type = hdfs__pb2._DATANODEINFOPROTO
_OPREPLACEBLOCKPROTO.fields_by_name['header'].message_type = _BASEHEADERPROTO
_OPREPLACEBLOCKPROTO.fields_by_name['source'].message_type = hdfs__pb2._DATANODEINFOPROTO
_OPCOPYBLOCKPROTO.fields_by_name['header'].message_type = _BASEHEADERPROTO
_OPBLOCKCHECKSUMPROTO.fields_by_name['header'].message_type = _BASEHEADERPROTO
_OPREQUESTSHORTCIRCUITACCESSPROTO.fields_by_name['header'].message_type = _BASEHEADERPROTO
_PIPELINEACKPROTO.fields_by_name['status'].enum_type = _STATUS
_READOPCHECKSUMINFOPROTO.fields_by_name['checksum'].message_type = _CHECKSUMPROTO
_BLOCKOPRESPONSEPROTO.fields_by_name['status'].enum_type = _STATUS
_BLOCKOPRESPONSEPROTO.fields_by_name['checksumResponse'].message_type = _OPBLOCKCHECKSUMRESPONSEPROTO
_BLOCKOPRESPONSEPROTO.fields_by_name['readOpChecksumInfo'].message_type = _READOPCHECKSUMINFOPROTO
_CLIENTREADSTATUSPROTO.fields_by_name['status'].enum_type = _STATUS
_DNTRANSFERACKPROTO.fields_by_name['status'].enum_type = _STATUS
_OPBLOCKCHECKSUMRESPONSEPROTO.fields_by_name['crcType'].enum_type = hdfs__pb2._CHECKSUMTYPEPROTO
DESCRIPTOR.message_types_by_name['DataTransferEncryptorMessageProto'] = _DATATRANSFERENCRYPTORMESSAGEPROTO
DESCRIPTOR.message_types_by_name['BaseHeaderProto'] = _BASEHEADERPROTO
DESCRIPTOR.message_types_by_name['ClientOperationHeaderProto'] = _CLIENTOPERATIONHEADERPROTO
DESCRIPTOR.message_types_by_name['CachingStrategyProto'] = _CACHINGSTRATEGYPROTO
DESCRIPTOR.message_types_by_name['OpReadBlockProto'] = _OPREADBLOCKPROTO
DESCRIPTOR.message_types_by_name['ChecksumProto'] = _CHECKSUMPROTO
DESCRIPTOR.message_types_by_name['OpWriteBlockProto'] = _OPWRITEBLOCKPROTO
DESCRIPTOR.message_types_by_name['OpTransferBlockProto'] = _OPTRANSFERBLOCKPROTO
DESCRIPTOR.message_types_by_name['OpReplaceBlockProto'] = _OPREPLACEBLOCKPROTO
DESCRIPTOR.message_types_by_name['OpCopyBlockProto'] = _OPCOPYBLOCKPROTO
DESCRIPTOR.message_types_by_name['OpBlockChecksumProto'] = _OPBLOCKCHECKSUMPROTO
DESCRIPTOR.message_types_by_name['OpRequestShortCircuitAccessProto'] = _OPREQUESTSHORTCIRCUITACCESSPROTO
DESCRIPTOR.message_types_by_name['PacketHeaderProto'] = _PACKETHEADERPROTO
DESCRIPTOR.message_types_by_name['PipelineAckProto'] = _PIPELINEACKPROTO
DESCRIPTOR.message_types_by_name['ReadOpChecksumInfoProto'] = _READOPCHECKSUMINFOPROTO
DESCRIPTOR.message_types_by_name['BlockOpResponseProto'] = _BLOCKOPRESPONSEPROTO
DESCRIPTOR.message_types_by_name['ClientReadStatusProto'] = _CLIENTREADSTATUSPROTO
DESCRIPTOR.message_types_by_name['DNTransferAckProto'] = _DNTRANSFERACKPROTO
DESCRIPTOR.message_types_by_name['OpBlockChecksumResponseProto'] = _OPBLOCKCHECKSUMRESPONSEPROTO
DESCRIPTOR.enum_types_by_name['Status'] = _STATUS

DataTransferEncryptorMessageProto = _reflection.GeneratedProtocolMessageType('DataTransferEncryptorMessageProto', (_message.Message,), dict(
  DESCRIPTOR = _DATATRANSFERENCRYPTORMESSAGEPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.DataTransferEncryptorMessageProto)
  ))
_sym_db.RegisterMessage(DataTransferEncryptorMessageProto)

BaseHeaderProto = _reflection.GeneratedProtocolMessageType('BaseHeaderProto', (_message.Message,), dict(
  DESCRIPTOR = _BASEHEADERPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.BaseHeaderProto)
  ))
_sym_db.RegisterMessage(BaseHeaderProto)

ClientOperationHeaderProto = _reflection.GeneratedProtocolMessageType('ClientOperationHeaderProto', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTOPERATIONHEADERPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.ClientOperationHeaderProto)
  ))
_sym_db.RegisterMessage(ClientOperationHeaderProto)

CachingStrategyProto = _reflection.GeneratedProtocolMessageType('CachingStrategyProto', (_message.Message,), dict(
  DESCRIPTOR = _CACHINGSTRATEGYPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.CachingStrategyProto)
  ))
_sym_db.RegisterMessage(CachingStrategyProto)

OpReadBlockProto = _reflection.GeneratedProtocolMessageType('OpReadBlockProto', (_message.Message,), dict(
  DESCRIPTOR = _OPREADBLOCKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpReadBlockProto)
  ))
_sym_db.RegisterMessage(OpReadBlockProto)

ChecksumProto = _reflection.GeneratedProtocolMessageType('ChecksumProto', (_message.Message,), dict(
  DESCRIPTOR = _CHECKSUMPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.ChecksumProto)
  ))
_sym_db.RegisterMessage(ChecksumProto)

OpWriteBlockProto = _reflection.GeneratedProtocolMessageType('OpWriteBlockProto', (_message.Message,), dict(
  DESCRIPTOR = _OPWRITEBLOCKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpWriteBlockProto)
  ))
_sym_db.RegisterMessage(OpWriteBlockProto)

OpTransferBlockProto = _reflection.GeneratedProtocolMessageType('OpTransferBlockProto', (_message.Message,), dict(
  DESCRIPTOR = _OPTRANSFERBLOCKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpTransferBlockProto)
  ))
_sym_db.RegisterMessage(OpTransferBlockProto)

OpReplaceBlockProto = _reflection.GeneratedProtocolMessageType('OpReplaceBlockProto', (_message.Message,), dict(
  DESCRIPTOR = _OPREPLACEBLOCKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpReplaceBlockProto)
  ))
_sym_db.RegisterMessage(OpReplaceBlockProto)

OpCopyBlockProto = _reflection.GeneratedProtocolMessageType('OpCopyBlockProto', (_message.Message,), dict(
  DESCRIPTOR = _OPCOPYBLOCKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpCopyBlockProto)
  ))
_sym_db.RegisterMessage(OpCopyBlockProto)

OpBlockChecksumProto = _reflection.GeneratedProtocolMessageType('OpBlockChecksumProto', (_message.Message,), dict(
  DESCRIPTOR = _OPBLOCKCHECKSUMPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpBlockChecksumProto)
  ))
_sym_db.RegisterMessage(OpBlockChecksumProto)

OpRequestShortCircuitAccessProto = _reflection.GeneratedProtocolMessageType('OpRequestShortCircuitAccessProto', (_message.Message,), dict(
  DESCRIPTOR = _OPREQUESTSHORTCIRCUITACCESSPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpRequestShortCircuitAccessProto)
  ))
_sym_db.RegisterMessage(OpRequestShortCircuitAccessProto)

PacketHeaderProto = _reflection.GeneratedProtocolMessageType('PacketHeaderProto', (_message.Message,), dict(
  DESCRIPTOR = _PACKETHEADERPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.PacketHeaderProto)
  ))
_sym_db.RegisterMessage(PacketHeaderProto)

PipelineAckProto = _reflection.GeneratedProtocolMessageType('PipelineAckProto', (_message.Message,), dict(
  DESCRIPTOR = _PIPELINEACKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.PipelineAckProto)
  ))
_sym_db.RegisterMessage(PipelineAckProto)

ReadOpChecksumInfoProto = _reflection.GeneratedProtocolMessageType('ReadOpChecksumInfoProto', (_message.Message,), dict(
  DESCRIPTOR = _READOPCHECKSUMINFOPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.ReadOpChecksumInfoProto)
  ))
_sym_db.RegisterMessage(ReadOpChecksumInfoProto)

BlockOpResponseProto = _reflection.GeneratedProtocolMessageType('BlockOpResponseProto', (_message.Message,), dict(
  DESCRIPTOR = _BLOCKOPRESPONSEPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.BlockOpResponseProto)
  ))
_sym_db.RegisterMessage(BlockOpResponseProto)

ClientReadStatusProto = _reflection.GeneratedProtocolMessageType('ClientReadStatusProto', (_message.Message,), dict(
  DESCRIPTOR = _CLIENTREADSTATUSPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.ClientReadStatusProto)
  ))
_sym_db.RegisterMessage(ClientReadStatusProto)

DNTransferAckProto = _reflection.GeneratedProtocolMessageType('DNTransferAckProto', (_message.Message,), dict(
  DESCRIPTOR = _DNTRANSFERACKPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.DNTransferAckProto)
  ))
_sym_db.RegisterMessage(DNTransferAckProto)

OpBlockChecksumResponseProto = _reflection.GeneratedProtocolMessageType('OpBlockChecksumResponseProto', (_message.Message,), dict(
  DESCRIPTOR = _OPBLOCKCHECKSUMRESPONSEPROTO,
  __module__ = 'datatransfer_pb2'
  # @@protoc_insertion_point(class_scope:hadoop.hdfs.OpBlockChecksumResponseProto)
  ))
_sym_db.RegisterMessage(OpBlockChecksumResponseProto)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n%org.apache.hadoop.hdfs.protocol.protoB\022DataTransferProtos\240\001\001'))
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Copyright (c) 2009 Las Cumbres Observatory (www.lcogt.net)
# Copyright (c) 2010 Jan Dittberner
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
'''
channel.py - Socket implementation of Google's Protocol Buffers RPC
service interface.

This package contains classes providing a socket implementation of the
RPCChannel abstract class.

Original Authors: Martin Norbury (mnorbury@lcogt.net)
         Eric Saunders (esaunders@lcogt.net)
         Jan Dittberner (jan@dittberner.info)

May 2009, Nov 2010

Modified for snakebite: Wouter de Bie (wouter@spotify.com)

May 2012

'''

# python 3 support
from __future__ import absolute_import, print_function, division

# Standard library imports
import socket
import os
import math

# Third party imports
from google.protobuf.service import RpcChannel

# Protobuf imports
from snakebite.protobuf.RpcHeader_pb2 import RpcRequestHeaderProto, RpcResponseHeaderProto
from snakebite.protobuf.IpcConnectionContext_pb2 import IpcConnectionContextProto
from snakebite.protobuf.ProtobufRpcEngine_pb2 import RequestHeaderProto
from snakebite.protobuf.datatransfer_pb2 import OpReadBlockProto, BlockOpResponseProto, PacketHeaderProto, ClientReadStatusProto

from snakebite.platformutils import get_current_username
from snakebite.formatter import format_bytes
from snakebite.errors import RequestError, TransientException, FatalException
from snakebite.crc32c import crc
from snakebite.compat import range, py_2
from snakebite import logger

import google.protobuf.internal.encoder as encoder
import google.protobuf.internal.decoder as decoder

# Module imports
import logging
import struct
import uuid

_kerberos_available = False
try:
    from snakebite.rpc_sasl import SaslRpcClient
    from snakebite.kerberos import Kerberos
    _kerberos_available = True
except ImportError:
    pass

# Configure package logging
log = logger.getLogger(__name__)


def log_protobuf_message(header, message):
    log.debug("%s:\n\n\033[92m%s\033[0m" % (header, message))


def get_delimited_message_bytes(byte_stream, nr=4):
    ''' Parse a delimited protobuf message. This is done by first getting a protobuf varint from
    the stream that represents the length of the message, then reading that amount of
    from the message and then parse it.
    Since the int can be represented as max 4 bytes, first get 4 bytes and try to decode.
    The decoder returns the value and the position where the value was found, so we need
    to rewind the buffer to the position, because the remaining bytes belong to the message
    after.
    '''

    (length, pos) = decoder._DecodeVarint32(byte_stream.read(nr), 0)
    if log.getEffectiveLevel() == logging.DEBUG:
        log.debug("Delimited message length (pos %d): %d" % (pos, length))

    delimiter_bytes = nr - pos

    byte_stream.rewind(delimiter_bytes)
    message_bytes = byte_stream.read(length)
    if log.getEffectiveLevel() == logging.DEBUG:
        log.debug("Delimited message bytes (%d): %s" % (len(message_bytes), format_bytes(message_bytes)))

    total_len = length + pos
    return (total_len, message_bytes)


class RpcBufferedReader(object):
    '''Class that wraps a socket and provides some utility methods for reading
    and rewinding of the buffer. This comes in handy when reading protobuf varints.
    '''
    MAX_READ_ATTEMPTS = 100

    def __init__(self, socket):
        self.socket = socket
        self.reset()

    def read(self, n):
        '''Reads n bytes into the internal buffer'''
        bytes_wanted = n - self.buffer_length + self.pos + 1
        if bytes_wanted > 0:
            self._buffer_bytes(bytes_wanted)

        end_pos = self.pos + n
        ret = self.buffer[self.pos + 1:end_pos + 1]
        self.pos = end_pos
        return ret

    def _buffer_bytes(self, n):
        to_read = n
        for _ in range(self.MAX_READ_ATTEMPTS):
            bytes_read = self.socket.recv(to_read)
            self.buffer += bytes_read
            to_read -= len(bytes_read)
            if to_read == 0:
                log.debug("Bytes read: %d, total: %d" % (len(bytes_read), self.buffer_length))
                return n
        if len(bytes_read) < n:
            # we'd like to distinguish transient (e.g. network-related) problems
            # note: but this error could also be a logic error
            raise TransientException("RpcBufferedReader only managed to read %s out of %s bytes" % (len(bytes_read), n))

    def rewind(self, places):
        '''Rewinds the current buffer to a position. Needed for reading varints,
        because we might read bytes that belong to the stream after the varint.
        '''
        log.debug("Rewinding pos %d with %d places" % (self.pos, places))
        self.pos -= places
        log.debug("Reset buffer to pos %d" % self.pos)

    def reset(self):
        self.buffer = b""
        self.pos = -1  # position of last byte read

    @property
    def buffer_length(self):
        '''Returns the length of the current buffer.'''
        return len(self.buffer)


class SocketRpcChannel(RpcChannel):
    ERROR_BYTES = 18446744073709551615
    RPC_HEADER = b"hrpc"
    RPC_SERVICE_CLASS = 0x00
    AUTH_PROTOCOL_NONE = 0x00
    AUTH_PROTOCOL_SASL = 0xDF
    RPC_PROTOCOL_BUFFFER = 0x02


    '''Socket implementation of an RpcChannel.
    '''

    def __init__(self, host, port, version, effective_user=None, use_sasl=False, hdfs_namenode_principal=None,
                 sock_connect_timeout=10000, sock_request_timeout=10000):
        '''SocketRpcChannel to connect to a socket server on a user defined port.
           It possible to define version and effective user for the communication.'''
        self.host = host
        self.port = port
        self.sock = None
        self.call_id = -3  # First time (when the connection context is sent, the call_id should be -3, otherwise start with 0 and increment)
        self.version = version
        self.client_id = str(uuid.uuid4()).encode("utf-8")
        self.use_sasl = use_sasl
        self.hdfs_namenode_principal = hdfs_namenode_principal
        if self.use_sasl:
            if not _kerberos_available:
                raise FatalException("Kerberos libs not found. Please install snakebite using 'pip install snakebite[kerberos]'")

            kerberos = Kerberos()
            self.effective_user = effective_user or kerberos.user_principal()
        else: 
            self.effective_user = effective_user or get_current_username()
        self.sock_connect_timeout = sock_connect_timeout
        self.sock_request_timeout = sock_request_timeout

    def validate_request(self, request):
        '''Validate the client request against the protocol file.'''

        # Check the request is correctly initialized
        if not request.IsInitialized():
            raise FatalException("Client request (%s) is missing mandatory fields" % type(request))

    def get_connection(self, host, port):
        '''Open a socket connection to a given host and port and writes the Hadoop header
        The Hadoop RPC protocol looks like this when creating a connection:

        +---------------------------------------------------------------------+
        |  Header, 4 bytes ("hrpc")                                           |
        +---------------------------------------------------------------------+
        |  Version, 1 byte (default verion 9)                                 |
        +---------------------------------------------------------------------+
        |  RPC service class, 1 byte (0x00)                                   |
        +---------------------------------------------------------------------+
        |  Auth protocol, 1 byte (Auth method None = 0)                       |
        +---------------------------------------------------------------------+
        |  Length of the RpcRequestHeaderProto  + length of the               |
        |  of the IpcConnectionContextProto (4 bytes/32 bit int)              |
        +---------------------------------------------------------------------+
        |  Serialized delimited RpcRequestHeaderProto                         |
        +---------------------------------------------------------------------+
        |  Serialized delimited IpcConnectionContextProto                     |
        +---------------------------------------------------------------------+
        '''

        log.debug("############## CONNECTING ##############")
        # Open socket
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        self.sock.settimeout(self.sock_connect_timeout / 1000)
        # Connect socket to server - defined by host and port arguments
        self.sock.connect((host, port))
        self.sock.settimeout(self.sock_request_timeout / 1000)

        # Send RPC headers
        self.write(self.RPC_HEADER)                             # header
        self.write(struct.pack('B', self.version))              # version
        self.write(struct.pack('B', self.RPC_SERVICE_CLASS))    # RPC service class
        if self.use_sasl:
            self.write(struct.pack('B', self.AUTH_PROTOCOL_SASL))   # serialization type (protobuf = 0xDF)
        else:
            self.write(struct.pack('B', self.AUTH_PROTOCOL_NONE))   # serialization type (protobuf = 0)

        if self.use_sasl:
            self.sasl = SaslRpcClient(self, hdfs_namenode_principal=self.hdfs_namenode_principal)
            sasl_connected = self.sasl.connect()
            if not sasl_connected:
                raise TransientException("SASL is configured, but cannot get connected")

        rpc_header = self.create_rpc_request_header()
        context = self.create_connection_context()

        header_length = len(rpc_header) + encoder._VarintSize(len(rpc_header)) +len(context) + encoder._VarintSize(len(context))

        if log.getEffectiveLevel() == logging.DEBUG:
            log.debug("Header length: %s (%s)" % (header_length, format_bytes(struct.pack('!I', header_length))))

        self.write(struct.pack('!I', header_length))

        self.write_delimited(rpc_header)
        self.write_delimited(context)
    
    def write(self, data):
        if log.getEffectiveLevel() == logging.DEBUG:
            log.debug("Sending: %s", format_bytes(data))
        self.sock.send(data)

    def write_delimited(self, data):
        self.write(encoder._VarintBytes(len(data)))
        self.write(data)

    def create_rpc_request_header(self):
        '''Creates and serializes a delimited RpcRequestHeaderProto message.'''
        rpcheader = RpcRequestHeaderProto()
        rpcheader.rpcKind = 2  # rpcheaderproto.RpcKindProto.Value('RPC_PROTOCOL_BUFFER')
        rpcheader.rpcOp = 0  # rpcheaderproto.RpcPayloadOperationProto.Value('RPC_FINAL_PACKET')
        rpcheader.callId = self.call_id
        rpcheader.retryCount = -1
        rpcheader.clientId = self.client_id[0:16]

        if self.call_id == -3:
            self.call_id = 0
        else:
            self.call_id += 1

        # Serialize delimited
        s_rpcHeader = rpcheader.SerializeToString()
        log_protobuf_message("RpcRequestHeaderProto (len: %d)" % (len(s_rpcHeader)), rpcheader)
        return s_rpcHeader

    def create_connection_context(self):
        '''Creates and seriazlies a IpcConnectionContextProto (not delimited)'''
        context = IpcConnectionContextProto()
        context.userInfo.effectiveUser = self.effective_user
        context.protocol = "org.apache.hadoop.hdfs.protocol.ClientProtocol"

        s_context = context.SerializeToString()
        log_protobuf_message("RequestContext (len: %d)" % len(s_context), context)
        return s_context

    def send_rpc_message(self, method, request):
        '''Sends a Hadoop RPC request to the NameNode.

        The IpcConnectionContextProto, RpcPayloadHeaderProto and HadoopRpcRequestProto
        should already be serialized in the right way (delimited or not) before
        they are passed in this method.

        The Hadoop RPC protocol looks like this for sending requests:

        When sending requests
        +---------------------------------------------------------------------+
        |  Length of the next three parts (4 bytes/32 bit int)                |
        +---------------------------------------------------------------------+
        |  Delimited serialized RpcRequestHeaderProto (varint len + header)   |
        +---------------------------------------------------------------------+
        |  Delimited serialized RequestHeaderProto (varint len + header)      |
        +---------------------------------------------------------------------+
        |  Delimited serialized Request (varint len + request)                |
        +---------------------------------------------------------------------+
        '''
        log.debug("############## SENDING ##############")

        #0. RpcRequestHeaderProto
        rpc_request_header = self.create_rpc_request_header()
        #1. RequestHeaderProto
        request_header = self.create_request_header(method)
        #2. Param
        param = request.SerializeToString()
        if log.getEffectiveLevel() == logging.DEBUG:
            log_protobuf_message("Request", request)

        rpc_message_length = len(rpc_request_header) + encoder._VarintSize(len(rpc_request_header)) + \
                             len(request_header) + encoder._VarintSize(len(request_header)) + \
                             len(param) + encoder._VarintSize(len(param))

        if log.getEffectiveLevel() == logging.DEBUG:
            log.debug("RPC message length: %s (%s)" % (rpc_message_length, format_bytes(struct.pack('!I', rpc_message_length))))

        if self.use_sasl and self.sasl.use_wrap():
            log.debug("SASL QOP requested, wrapping RPC message.")
            self.sasl.wrap(
                struct.pack('!I', rpc_message_length) +
                encoder._VarintBytes(len(rpc_request_header)) + rpc_request_header +
                encoder._VarintBytes(len(request_header)) + request_header +
                encoder._VarintBytes(len(param)) + param)
        else:
            self.write(struct.pack('!I', rpc_message_length))
            self.write_delimited(rpc_request_header)
            self.write_delimited(request_header)
            self.write_delimited(param)

    def create_request_header(self, method):
        header = RequestHeaderProto()
        header.methodName = method.name
        header.declaringClassProtocolName = "org.apache.hadoop.hdfs.protocol.ClientProtocol"
        header.clientProtocolVersion = 1

        s_header = header.SerializeToString()
        log_protobuf_message("RequestHeaderProto (len: %d)" % len(s_header), header)
        return s_header

    def recv_rpc_message(self):
        '''Handle reading an RPC reply from the server. This is done by wrapping the
        socket in a RcpBufferedReader that allows for rewinding of the buffer stream.
        '''
        log.debug("############## RECVING ##############")
        byte_stream = RpcBufferedReader(self.sock)
        return byte_stream

    def get_length(self, byte_stream):
        ''' In Hadoop protobuf RPC, some parts of the stream are delimited with protobuf varint,
        while others are delimited with 4 byte integers. This reads 4 bytes from the byte stream
        and retruns the length of the delimited part that follows, by unpacking the 4 bytes
        and returning the first element from a tuple. The tuple that is returned from struc.unpack()
        only contains one element.
        '''
        length = struct.unpack("!i", byte_stream.read(4))[0]
        log.debug("4 bytes delimited part length: %d" % length)
        return length

    def parse_response(self, byte_stream, response_class):
        '''Parses a Hadoop RPC response.

        The RpcResponseHeaderProto contains a status field that marks SUCCESS or ERROR.
        The Hadoop RPC protocol looks like the diagram below for receiving SUCCESS requests.
        +-----------------------------------------------------------+
        |  Length of the RPC resonse (4 bytes/32 bit int)           |
        +-----------------------------------------------------------+
        |  Delimited serialized RpcResponseHeaderProto              |
        +-----------------------------------------------------------+
        |  Serialized delimited RPC response                        |
        +-----------------------------------------------------------+

        In case of an error, the header status is set to ERROR and the error fields are set.
        '''

        log.debug("############## PARSING ##############")
        log.debug("Payload class: %s" % response_class)

        # Read first 4 bytes to get the total length
        len_bytes = byte_stream.read(4)
        total_length = struct.unpack("!I", len_bytes)[0]
        log.debug("Total response length: %s" % total_length)

        header = RpcResponseHeaderProto()
        (header_len, header_bytes) = get_delimited_message_bytes(byte_stream)

        log.debug("Header read %d" % header_len)
        header.ParseFromString(header_bytes)
        log_protobuf_message("RpcResponseHeaderProto", header)

        if header.status == 0:
            log.debug("header: %s, total: %s" % (header_len, total_length))
            if header_len >= total_length:
                return
            response = response_class()
            response_bytes = get_delimited_message_bytes(byte_stream, total_length - header_len)[1]
            if len(response_bytes) > 0:
                response.ParseFromString(response_bytes)
                if log.getEffectiveLevel() == logging.DEBUG:
                    log_protobuf_message("Response", response)
                return response
        else:
            self.handle_error(header)

    def unwrap_response(self, response_bytes, response_class):
        '''Parse a Hadoop RPC response from a byte array.

        This is useful when SASL QOP is requested and the Hadoop RPC
        response is wrapped/encrypted in a SASL RPC message.
        '''

        # Read the first 4 bytes to get the total length
        len_bytes = response_bytes[:4]
        total_length = struct.unpack("!I", len_bytes)[0]
        log.debug("Total response length: %s" % total_length)
        # Read the next 4 bytes to get the length of
        # the RPC Response header.
        # Caveat: the value can take up to 4 bytes, but usually
        # it may be less.
        (header_length, pos) = decoder._DecodeVarint32(response_bytes[4:8], 0)
        log.debug("Delimited message length (pos %d): %d" % (pos, header_length))
        current = 4 + pos
        header = RpcResponseHeaderProto()
        header.ParseFromString(response_bytes[current:current+header_length])
        log_protobuf_message("RpcResponseHeaderProto", header)
        current += header_length
        # Read the next 4 bytes to get the length of the RPC Response body.
        # Caveat: the value can take up to 4 bytes, but usually
        # it may be less.
        (msg_length, pos2) = decoder._DecodeVarint32(response_bytes[current:current+4], 0)
        log.debug("Delimited message length (pos %d): %d" % (pos2, msg_length))
        response = response_class()
        current += pos2
        response.ParseFromString(response_bytes[current:])
        if log.getEffectiveLevel() == logging.DEBUG:
            log_protobuf_message("Response", response)
        return response



    def handle_error(self, header):
        raise RequestError("\n".join([header.exceptionClassName, header.errorMsg]))

    def close_socket(self):
        '''Closes the socket and resets the channel.'''
        log.debug("Closing socket")
        if self.sock:
            try:
                self.sock.close()
            except:
                pass

            self.sock = None

    def CallMethod(self, method, controller, request, response_class, done):
        '''Call the RPC method. The naming doesn't confirm PEP8, since it's
        a method called by protobuf
        '''
        try:
            self.validate_request(request)

            if not self.sock:
                self.get_connection(self.host, self.port)

            self.send_rpc_message(method, request)
            if self.use_sasl and self.sasl.use_wrap():
                log.debug("SASL QOP requested, unwrapping RPC message.")
                response_bytes = self.sasl.unwrap()
                return self.unwrap_response(response_bytes, response_class)
            else:
                byte_stream = self.recv_rpc_message()
                return self.parse_response(byte_stream, response_class)
        except RequestError:  # Raise a request error, but don't close the socket
            raise
        except Exception:  # All other errors close the socket
            self.close_socket()
            raise


class DataXceiverChannel(object):
    # For internal reading: should be larger than bytes_per_chunk
    LOAD_SIZE = 16000.0

    # Op codes
    WRITE_BLOCK = 80
    READ_BLOCK = 81
    READ_METADATA = 82
    REPLACE_BLOCK = 83
    COPY_BLOCK = 84
    BLOCK_CHECKSUM = 85
    TRANSFER_BLOCK = 86

    # Checksum types
    CHECKSUM_NULL = 0
    CHECKSUM_CRC32 = 1
    CHECKSUM_CRC32C = 2
    CHECKSUM_DEFAULT = 3
    CHECKSUM_MIXED = 4

    MAX_READ_ATTEMPTS = 100

    def __init__(self, host, port):
        self.host, self.port = host, port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def connect(self):
        try:
            self.sock.connect((self.host, self.port))
            log.debug("%s connected to DataNode" % self)
            return True
        except Exception:
            log.debug("%s connection to DataNode failed" % self)
            return False

    def _close_socket(self):
        self.sock.close()

    def _read_bytes(self, n, depth=0):
        if depth > self.MAX_READ_ATTEMPTS:
            raise TransientException("Tried to read %d more bytes, but failed after %d attempts" % (n, self.MAX_READ_ATTEMPTS))

        _bytes = self.sock.recv(n)
        if len(_bytes) < n:
            left = n - len(_bytes)
            depth += 1
            _bytes += self._read_bytes(left, depth)
        return _bytes

    def write(self, data):
        if log.getEffectiveLevel() == logging.DEBUG:
            log.debug("Sending: %s", format_bytes(data))
        self.sock.send(data)

    def write_delimited(self, data):
        self.write(encoder._VarintBytes(len(data)))
        self.write(data)

    def readBlock(self, length, pool_id, block_id, generation_stamp, offset, block_token, check_crc):
        '''Send a read request to given block. If we receive a successful response,
        we start reading packets.

        Send read request:
        +---------------------------------------------------------------------+
        |  Data Transfer Protocol Version, 2 bytes                            |
        +---------------------------------------------------------------------+
        |  Op code, 1 byte (READ_BLOCK = 81)                                  |
        +---------------------------------------------------------------------+
        |  Delimited serialized OpReadBlockProto (varint len + request)       |
        +---------------------------------------------------------------------+

        Receive response:
        +---------------------------------------------------------------------+
        |  Delimited BlockOpResponseProto (varint len + response)             |
        +---------------------------------------------------------------------+

        Start reading packets. Each packet has the following structure:
        +---------------------------------------------------------------------+
        |  Packet length (4 bytes/32 bit int)                                 |
        +---------------------------------------------------------------------+
        |  Serialized size of header, 2 bytes                                 |
        +---------------------------------------------------------------------+
        |  Packet Header Proto                                                |
        +---------------------------------------------------------------------+
        |  x checksums, 4 bytes each                                          |
        +---------------------------------------------------------------------+
        |  x chunks of payload data                                           |
        +---------------------------------------------------------------------+

        '''
        log.debug("%s sending readBlock request" % self)

        # Send version and opcode
        self.sock.send(struct.pack('>h', 28))
        self.sock.send(struct.pack('b', self.READ_BLOCK))
        length = length - offset

        # Create and send OpReadBlockProto message
        request = OpReadBlockProto()
        request.offset = offset
        request.len = length
        header = request.header
        header.clientName = "snakebite"
        base_header = header.baseHeader
        # TokenProto
        token = base_header.token
        token.identifier = block_token.identifier
        token.password = block_token.password
        token.kind = block_token.kind
        token.service = block_token.service
        # ExtendedBlockProto
        block = base_header.block
        block.poolId = pool_id
        block.blockId = block_id
        block.generationStamp = generation_stamp
        s_request = request.SerializeToString()
        log_protobuf_message("OpReadBlockProto:", request)
        self.write_delimited(s_request)

        byte_stream = RpcBufferedReader(self.sock)
        block_op_response_bytes = get_delimited_message_bytes(byte_stream)[1]

        block_op_response = BlockOpResponseProto()
        block_op_response.ParseFromString(block_op_response_bytes)
        log_protobuf_message("BlockOpResponseProto", block_op_response)

        checksum_type = block_op_response.readOpChecksumInfo.checksum.type
        bytes_per_chunk = block_op_response.readOpChecksumInfo.checksum.bytesPerChecksum
        log.debug("Checksum type: %s, bytesPerChecksum: %s" % (checksum_type, bytes_per_chunk))
        if checksum_type in [self.CHECKSUM_NULL]:
            checksum_len = 0
        elif checksum_type in [self.CHECKSUM_CRC32C, self.CHECKSUM_CRC32]:
            checksum_len = 4
        else:
            raise FatalException("Checksum type %s not implemented" % checksum_type)

        total_read = 0
        if block_op_response.status == 0:  # datatransfer_proto.Status.Value('SUCCESS')
            while total_read < length:
                log.debug("== Reading next packet")

                packet_len = struct.unpack("!I", byte_stream.read(4))[0]
                log.debug("Packet length: %s", packet_len)

                serialized_size = struct.unpack("!H", byte_stream.read(2))[0]
                log.debug("Serialized size: %s", serialized_size)

                packet_header_bytes = byte_stream.read(serialized_size)
                packet_header = PacketHeaderProto()
                packet_header.ParseFromString(packet_header_bytes)
                log_protobuf_message("PacketHeaderProto", packet_header)

                data_len = packet_header.dataLen

                chunks_per_packet = int((data_len + bytes_per_chunk - 1) / bytes_per_chunk)
                log.debug("Nr of chunks: %d", chunks_per_packet)

                data_len = packet_len - 4 - chunks_per_packet * checksum_len
                log.debug("Payload len: %d", data_len)

                byte_stream.reset()

                # Collect checksums
                if check_crc and checksum_type != self.CHECKSUM_NULL:
                    checksums = []
                    for _ in range(0, chunks_per_packet):
                        checksum = self._read_bytes(checksum_len)
                        checksum = struct.unpack("!I", checksum)[0]
                        checksums.append(checksum)
                else:
                    self._read_bytes(checksum_len * chunks_per_packet)

                # We use a fixed size buffer (a "load") to read only a couple of chunks at once. 
                bytes_per_load = self.LOAD_SIZE - (self.LOAD_SIZE % bytes_per_chunk)
                chunks_per_load = int(bytes_per_load / bytes_per_chunk)
                loads_per_packet = int(math.ceil(bytes_per_chunk * chunks_per_packet / bytes_per_load))

                read_on_packet = 0
                for i in range(loads_per_packet):
                    load = b''
                    for j in range(chunks_per_load):
                        log.debug("Reading chunk %s in load %s:", j, i)
                        bytes_to_read = min(bytes_per_chunk, data_len - read_on_packet)
                        chunk = self._read_bytes(bytes_to_read)
                        if check_crc and checksum_type != self.CHECKSUM_NULL:
                            checksum_index = i * chunks_per_load + j
                            if checksum_index < len(checksums) and crc(chunk) != checksums[checksum_index]:
                                # it makes sense to retry, so TransientError
                                raise TransientException("Checksum doesn't match")
                        load += chunk
                        total_read += len(chunk)
                        read_on_packet += len(chunk)
                    yield load
           
            # Send ClientReadStatusProto message confirming successful read
            request = ClientReadStatusProto()
            request.status = 0  # SUCCESS
            log_protobuf_message("ClientReadStatusProto:", request)
            s_request = request.SerializeToString()
            self.write_delimited(s_request)
            self._close_socket()

    def __repr__(self):
        return "DataXceiverChannel<%s:%d>" % (self.host, self.port)



# python 3 support
from __future__ import absolute_import, print_function, division

import os
import logging
import xml.etree.ElementTree as ET
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from snakebite.namenode import Namenode

log = logging.getLogger(__name__)


class HDFSConfig(object):
    @classmethod
    def get_config_from_env(cls):
        """
        .. deprecated:: 2.5.3
        Gets configuration out of environment.

        Returns list of dicts - list of namenode representations
        """
        core_path = os.path.join(os.environ['HADOOP_HOME'], 'conf', 'core-site.xml')
        core_configs = cls.read_core_config(core_path)

        hdfs_path = os.path.join(os.environ['HADOOP_HOME'], 'conf', 'hdfs-site.xml')
        hdfs_configs = cls.read_hdfs_config(hdfs_path)

        if (not core_configs) and (not hdfs_configs):
            raise Exception("No config found in %s nor in %s" % (core_path, hdfs_path))

        configs = {
            'use_trash': hdfs_configs.get('use_trash', core_configs.get('use_trash', False)),
            'use_sasl': core_configs.get('use_sasl', False),
            'hdfs_namenode_principal': hdfs_configs.get('hdfs_namenode_principal', None),
            'namenodes': hdfs_configs.get('namenodes', []) or core_configs.get('namenodes', [])
        }

        return configs

    @staticmethod
    def read_hadoop_config(hdfs_conf_path):
        if os.path.exists(hdfs_conf_path):
            try:
                tree = ET.parse(hdfs_conf_path)
            except:
                log.error("Unable to parse %s" % hdfs_conf_path)
                return
            root = tree.getroot()
            for p in root.findall("./property"):
                yield p

    @classmethod
    def read_core_config(cls, core_site_path):
        configs = {}

        namenodes = []
        for property in cls.read_hadoop_config(core_site_path):

            # fs.default.name is the key name for the file system on EMR clusters
            if property.findall('name')[0].text in ('fs.defaultFS', 'fs.default.name'):
                parse_result = urlparse(property.findall('value')[0].text)
                log.debug("Got namenode '%s' from %s" % (parse_result.geturl(), core_site_path))

                namenodes.append({"namenode": parse_result.hostname,
                               "port": parse_result.port if parse_result.port
                                                         else Namenode.DEFAULT_PORT})

            if property.findall('name')[0].text == 'fs.trash.interval':
                configs['use_trash'] = True

            if property.findall('name')[0].text == 'hadoop.security.authentication':
                log.debug("Got hadoop.security.authentication '%s'" % (property.findall('value')[0].text))
                if property.findall('value')[0].text == 'kerberos':
                    configs['use_sasl'] = True
                else:
                    configs['use_sasl'] = False

        if namenodes: 
            configs['namenodes'] = namenodes

        return configs

    @classmethod
    def read_hdfs_config(cls, hdfs_site_path):
        configs = {}

        namenodes = []
        for property in cls.read_hadoop_config(hdfs_site_path):
            if property.findall('name')[0].text.startswith("dfs.namenode.rpc-address"):
                parse_result = urlparse("//" + property.findall('value')[0].text)
                log.debug("Got namenode '%s' from %s" % (parse_result.geturl(), hdfs_site_path))
                namenodes.append({"namenode": parse_result.hostname,
                                "port": parse_result.port if parse_result.port
                                                          else Namenode.DEFAULT_PORT})

            if property.findall('name')[0].text == 'fs.trash.interval':
                configs['use_trash'] = True

            if property.findall('name')[0].text == 'dfs.namenode.kerberos.principal':
                log.debug("hdfs principal found: '%s'" % (property.findall('value')[0].text))
                configs['hdfs_namenode_principal'] = property.findall('value')[0].text

            if property.findall('name')[0].text == 'dfs.client.retry.max.attempts':
                configs['client_retries'] = int(property.findall('value')[0].text)

            if property.findall('name')[0].text == 'dfs.client.socket-timeout':
                configs['socket_timeout_millis'] = int(property.findall('value')[0].text)

            if property.findall('name')[0].text == 'dfs.client.failover.sleep.base.millis':
                configs['client_sleep_base_millis'] = int(property.findall('value')[0].text)

            if property.findall('name')[0].text == 'dfs.client.failover.sleep.max.millis':
                configs['client_sleep_max_millis'] = int(property.findall('value')[0].text)

            if property.findall('name')[0].text == 'dfs.client.failover.max.attempts':
                configs['failover_max_attempts'] = int(property.findall('value')[0].text)

            if property.findall('name')[0].text == 'dfs.client.use.datanode.hostname':
                configs['use_datanode_hostname'] = bool(property.findall('value')[0].text)

        if namenodes:
            configs['namenodes'] = namenodes

        return configs

    core_try_paths = ('/etc/hadoop/conf/core-site.xml',
                      '/usr/local/etc/hadoop/conf/core-site.xml',
                      '/usr/local/hadoop/conf/core-site.xml')

    hdfs_try_paths = ('/etc/hadoop/conf/hdfs-site.xml',
                      '/usr/local/etc/hadoop/conf/hdfs-site.xml',
                      '/usr/local/hadoop/conf/hdfs-site.xml')

    @classmethod
    def get_external_config(cls):
        if os.environ.get('HADOOP_HOME'):
            hdfs_path = os.path.join(os.environ['HADOOP_HOME'], 'conf', 'hdfs-site.xml')
            cls.hdfs_try_paths = (hdfs_path,) + cls.hdfs_try_paths
            core_path = os.path.join(os.environ['HADOOP_HOME'], 'conf', 'core-site.xml')
            cls.core_try_paths = (core_path,) + cls.core_try_paths
        if os.environ.get('HADOOP_CONF_DIR'):
            hdfs_path = os.path.join(os.environ['HADOOP_CONF_DIR'], 'hdfs-site.xml')
            cls.hdfs_try_paths = (hdfs_path,) + cls.hdfs_try_paths
            core_path = os.path.join(os.environ['HADOOP_CONF_DIR'], 'core-site.xml')
            cls.core_try_paths = (core_path,) + cls.core_try_paths

        # Try to find other paths
        core_configs = {}
        for core_conf_path in cls.core_try_paths:
            core_configs = cls.read_core_config(core_conf_path)
            if core_configs:
                break

        hdfs_configs = {}
        for hdfs_conf_path in cls.hdfs_try_paths:
            hdfs_configs = cls.read_hdfs_config(hdfs_conf_path)
            if hdfs_configs:
                break

        configs = {
            'use_trash': hdfs_configs.get('use_trash', core_configs.get('use_trash', False)),
            'use_sasl': core_configs.get('use_sasl', False),
            'hdfs_namenode_principal': hdfs_configs.get('hdfs_namenode_principal', None),
            'namenodes': hdfs_configs.get('namenodes', []) or core_configs.get('namenodes', []),
            'client_retries' : hdfs_configs.get('client_retries', 10),
            'client_sleep_base_millis' : hdfs_configs.get('client_sleep_base_millis', 500),
            'client_sleep_max_millis' : hdfs_configs.get('client_sleep_max_millis', 15000),
            'socket_timeout_millis' : hdfs_configs.get('socket_timeout_millis', 60000),
            'failover_max_attempts' : hdfs_configs.get('failover_max_attempts', 15),
            'use_datanode_hostname' : hdfs_configs.get('use_datanode_hostname', False)
        }

        return configs
